Package: phantasus
Title: Visual and interactive gene expression analysis
Version: 1.30.0
Authors@R: c(
    person("Maxim", "Kleverov", role = "aut"),        
    person("Daria", "Zenkova", role = "aut"),
    person("Vladislav", "Kamenev", role = "aut"),        
    person("Margarita", "Sablina", role = "ctb"),        
    person("Maxim", "Artyomov", role = "aut"),
    person("Alexey", "Sergushichev", email = "alsergbox@gmail.com", role = c("aut", "cre")))
Description: 
    Phantasus is a web-application for visual and interactive gene expression analysis.
    Phantasus is based on Morpheus – a web-based software for heatmap visualisation 
    and analysis, which was integrated with an R environment via OpenCPU API.
    Aside from basic visualization and filtering methods, R-based methods such 
    as k-means clustering, principal component analysis or differential expression analysis
    with limma package are supported.
URL: https://alserglab.wustl.edu/phantasus
BugReports: https://github.com/ctlab/phantasus/issues
Depends: R (>= 4.3)
biocViews: GeneExpression, GUI, Visualization, DataRepresentation,
        Transcriptomics, RNASeq, Microarray, Normalization, Clustering,
        DifferentialExpression, PrincipalComponent, ImmunoOncology
Imports: ggplot2, protolite, Biobase, GEOquery, Rook, htmltools,
        httpuv, jsonlite, limma, edgeR, opencpu, assertthat, methods,
        httr, rhdf5, utils, parallel, stringr, fgsea (>= 1.9.4),
        svglite, gtable, stats, Matrix, pheatmap, scales, ccaPP, grid,
        grDevices, AnnotationDbi, DESeq2, data.table, curl, apeglm,
        tidyr, config (>= 0.3.2), rhdf5client (>= 1.25.1), yaml, fs,
        phantasusLite, XML
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Suggests: testthat, BiocStyle, knitr, rmarkdown, org.Hs.eg.db,
        org.Mm.eg.db
VignetteBuilder: knitr
NeedsCompilation: no
Config/build/copy-method: link
git_url: https://git.bioconductor.org/packages/phantasus
git_branch: RELEASE_3_22
git_last_commit: 7182df1
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-31
Packaged: 2025-10-31 23:27:47 UTC; biocbuild
Author: Maxim Kleverov [aut],
  Daria Zenkova [aut],
  Vladislav Kamenev [aut],
  Margarita Sablina [ctb],
  Maxim Artyomov [aut],
  Alexey Sergushichev [aut, cre]
Maintainer: Alexey Sergushichev <alsergbox@gmail.com>
