# Generated by roxygen2: do not edit by hand

export(generatePreloadedSession)
export(getES)
export(getGDS)
export(getGSE)
export(getPhantasusConf)
export(read.gct)
export(reparseCachedESs)
export(servePhantasus)
export(setupPhantasus)
export(write.gct)
import(Biobase)
import(GEOquery)
import(Rook)
import(apeglm)
import(ccaPP)
import(data.table)
import(edgeR)
import(ggplot2)
import(grid)
import(gtable)
import(htmltools)
import(httpuv)
import(jsonlite)
import(limma)
import(opencpu)
import(rhdf5)
import(stats)
import(svglite)
importFrom(AnnotationDbi,columns)
importFrom(AnnotationDbi,keys)
importFrom(AnnotationDbi,loadDb)
importFrom(AnnotationDbi,mapIds)
importFrom(AnnotationDbi,species)
importFrom(fgsea,fgseaMultilevel)
importFrom(fgsea,plotEnrichment)
importFrom(ggplot2,ggtitle)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,svg)
importFrom(methods,new)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(tidyr,spread)
importFrom(utils,download.file)
importFrom(utils,getFromNamespace)
importFrom(utils,head)
importFrom(utils,object.size)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(utils,write.table)
