#' Example dataset
#'
#' Small slice from GSE27112-GPL6103 for
#'     runnable examples.
#'
#' @docType data
#'
#' @usage data(es)
#'
#' @examples
#' \dontrun{
#' data(es)
#' performKmeans(es, k = 2)
#' }
#'
"es"
