% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/appConfig.R
\name{createDockerConf}
\alias{createDockerConf}
\title{Creates default Docker conf file
Copies the package user.conf template to \code{user_conf_file} when the
file is absent. When it already exists, any keys present in the template
but missing from the existing file are added (e.g. \code{internal_ports}
on an upgrade from a pre-multiport installation).}
\usage{
createDockerConf(user_conf_file = confFile("user.conf"))
}
\arguments{
\item{user_conf_file}{Location of the user conf file to write.}
}
\description{
Creates default Docker conf file
Copies the package user.conf template to \code{user_conf_file} when the
file is absent. When it already exists, any keys present in the template
but missing from the existing file are added (e.g. \code{internal_ports}
on an upgrade from a pre-multiport installation).
}
\keyword{internal}
