% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phantasus-package.R
\docType{data}
\name{fgseaExample}
\alias{fgseaExample}
\title{Example pathway data.frame for fgsea tool}
\description{
Example pathway data.frame for fgsea tool
}
